from guillotina import configure
from guillotina.auth.validators import hash_password
from guillotina.event import notify
from guillotina.events import NewUserAdded
from guillotina.interfaces import IObjectAddedEvent
from guillotina_dbusers.content.users import IUser


@configure.subscriber(for_=(IUser, IObjectAddedEvent))
async def user_created(user, event):
    user.password = hash_password(user.password)

    # user has access to his own object by default
    # request = get_current_request()
    # roleperm = IPrincipalRoleManager(request.container)
    # roleperm.assign_role_to_principal('guillotina.Owner', user.id)

    await notify(NewUserAdded(user))



from guillotina import configure
from guillotina.api.service import Service
from guillotina.component import queryMultiAdapter
from guillotina.interfaces import IContainer, IResourceSerializeToJson
from guillotina.utils import get_authenticated_user


@configure.service(
    context=IContainer,
    name="@user_info",
    method="GET",
    permission="guillotina.Authenticated")
class Info(Service):

    async def __call__(self):
        user = get_authenticated_user(self.request)
        serializer = queryMultiAdapter((user, self.request), IResourceSerializeToJson)
        if serializer:
            data = await serializer()
        else:
            data = {}
        data.update({
            'id': user.id,
            'roles': user.roles,
            'groups': getattr(user, 'groups', [])
        })
        return data



# -*- coding: utf-8 -*-
from guillotina.tests.utils import get_container


async def test_content(dbusers_requester):
    async with await dbusers_requester as requester:
        container = await get_container(requester)
        users = await container.async_get('users')
        assert users.type_name == 'UserManager'
        groups = await container.async_get('groups')
        assert groups.type_name == 'GroupManager'



# -*- encoding: utf-8 -*-
from guillotina import configure, schema
from guillotina.content import Folder
from guillotina.interfaces import Allow, IFolder
from guillotina_dbusers import _


class IUserManager(IFolder):
    pass


class IUser(IFolder):

    username = schema.TextLine(
        title=_('Username'),
        required=False)

    email = schema.TextLine(
        title=_('Email'),
        required=False)

    name = schema.TextLine(
        title=_('Name'),
        required=False)

    password = schema.TextLine(
        title=_('Password'),
        required=False)

    user_groups = schema.List(
        title=_('Groups'),
        value_type=schema.TextLine(),
        required=False
    )

    user_roles = schema.List(
        title=_('Roles'),
        value_type=schema.TextLine(),
        required=False
    )

    user_permissions = schema.List(
        title=_('Permissions'),
        value_type=schema.TextLine(),
        required=False,
        default=[]
    )

    disabled = schema.Bool(
        title=_('Disabled'),
        default=False
    )

    properties = schema.Dict(
        required=False,
        default={}
    )


@configure.contenttype(
    type_name="User",
    schema=IUser,
    add_permission="guillotina.AddUser",
    behaviors=["guillotina.behaviors.dublincore.IDublinCore"])
class User(Folder):
    username = email = name = password = None
    disabled = False
    user_roles = ['guillotina.Member']
    user_groups = []
    user_permissions = {}
    properties = {}

    @property
    def roles(self):
        roles = {
            'guillotina.Authenticated': Allow
        }
        for role in getattr(self, 'user_roles', []) or []:
            roles[role] = Allow
        return roles

    @property
    def permissions(self):
        permissions = {}
        for permission in getattr(self, 'user_permissions', []) or []:
            permissions[permission] = Allow
        return permissions

    @property
    def groups(self):
        return getattr(self, 'user_groups', []) or []

    @property
    def _properties(self):
        return {}


@configure.contenttype(
    type_name="UserManager",
    schema=IUserManager,
    behaviors=["guillotina.behaviors.dublincore.IDublinCore"],
    allowed_types=["User"])
class UserManager(Folder):
    pass



# -*- coding: utf-8 -*-
from guillotina.tests.utils import get_container

import base64
import json


async def test_add_user(dbusers_requester):
    async with await dbusers_requester as requester:
        resp, status_code = await requester('GET', '/db/guillotina/users')
        resp, status_code = await requester(
            'POST',
            '/db/guillotina/users',
            data=json.dumps({
                "@type": "User",
                "name": "Foobar",
                "id": "foobar",
                "username": "foobar",
                "email": "foo@bar.com",
                "password": "password"
            })
        )
        assert status_code == 201

        container = await get_container(requester)
        users = await container.async_get('users')
        assert await users.async_contains('foobar')


async def test_user_auth(dbusers_requester):
    async with await dbusers_requester as requester:
        await requester(
            'POST',
            '/db/guillotina/users',
            data=json.dumps({
                "@type": "User",
                "name": "Foobar",
                "id": "foobar",
                "username": "foobar",
                "email": "foo@bar.com",
                "password": "password"
            })
        )
        # user should be able to add content to object
        resp, status_code = await requester(
            'POST',
            '/db/guillotina/users/foobar',
            data=json.dumps({
                "@type": "Item",
                "id": "foobaritem",
                "title": "foobar"
            }),
            token=base64.b64encode(b'foobar:password').decode('ascii')
        )
        container = await get_container(requester)
        users = await container.async_get('users')
        foobar = await users.async_get('foobar')
        assert await foobar.async_contains('foobaritem')


async def test_login(dbusers_requester):
    async with await dbusers_requester as requester:
        await requester(
            'POST',
            '/db/guillotina/users',
            data=json.dumps({
                "@type": "User",
                "name": "Foobar",
                "id": "foobar",
                "username": "foobar",
                "email": "foo@bar.com",
                "password": "password",
                "user_groups": ["Managers"]
            })
        )

        resp, status_code = await requester(
            'POST',
            '/db/guillotina/@login',
            data=json.dumps({
                "username": "foobar",
                "password": "password"
            })
        )
        assert status_code == 200

        # test using new auth token
        resp, status_code = await requester(
            'GET', '/db/guillotina/@addons',
            token=resp['token'],
            auth_type='Bearer'
        )
        assert status_code == 200


async def test_refresh(dbusers_requester):
    async with await dbusers_requester as requester:
        await requester(
            'POST',
            '/db/guillotina/users',
            data=json.dumps({
                "@type": "User",
                "name": "Foobar",
                "id": "foobar",
                "username": "foobar",
                "email": "foo@bar.com",
                "password": "password",
                "user_groups": ["Managers"]
            })
        )

        resp, status_code = await requester(
            'POST',
            '/db/guillotina/@login',
            data=json.dumps({
                "username": "foobar",
                "password": "password"
            })
        )
        assert status_code == 200

        resp, status_code = await requester(
            'POST', '/db/guillotina/@refresh_token',
            token=resp['token'],
            auth_type='Bearer'
        )
        assert status_code == 200
        assert 'token' in resp



# -*- coding: utf-8 -*-
from guillotina import configure
from guillotina.addons import Addon
from guillotina.content import create_content_in_container
from guillotina.interfaces import ILayers
from guillotina.utils import get_authenticated_user_id

USERS_LAYER = 'guillotina_dbusers.interfaces.IDBUsersLayer'


@configure.addon(
    name="dbusers",
    title="Guillotina DB Users")
class DBUsersAddon(Addon):

    @classmethod
    async def install(self, site, request):
        registry = request.container_settings
        registry.for_interface(ILayers)['active_layers'] |= {
            USERS_LAYER
        }
        user = get_authenticated_user_id(request)
        await create_content_in_container(
            site, 'UserManager', 'users',
            creators=(user,), title='Users')
        await create_content_in_container(
            site, 'GroupManager', 'groups',
            creators=(user,), title='Groups')

    @classmethod
    async def uninstall(self, site, request):
        registry = request.container_settings
        registry.for_interface(ILayers)['active_layers'] -= {
            USERS_LAYER
        }
        if await site.async_contains('users'):
            await site.async_del('users')
        if await site.async_contains('groups'):
            await site.async_del('groups')





class DBUserIdentifier(object):

    def __init__(self, request):
        self.request = request

    async def get_user(self, token):
        try:
            users = await self.request.container.async_get('users')
        except (AttributeError, KeyError):
            return

        user_ids = await users.async_keys()
        if token.get('id', '') in user_ids:
            user = await users.async_get(token.get('id', ''))
            if not user.disabled:
                return user



# -*- coding: utf-8 -*-
from guillotina import configure
from guillotina.i18n import MessageFactory

_ = MessageFactory('guillotina_dbusers')


app_settings = {
    "auth_user_identifiers": [
        "guillotina_dbusers.users.DBUserIdentifier"
    ]
}

configure.permission("guillotina.NotAuthenticated", "")
configure.permission("guillotina.Authenticated", "")
configure.permission("guillotina.AddUser", title="Add plone user")
configure.permission("guillotina.AddGroup", title="Add plone group")
configure.grant(permission="guillotina.AccessContent",
                role="guillotina.Anonymous")
configure.grant(permission="guillotina.NotAuthenticated",
                role="guillotina.Anonymous")
configure.grant(permission="guillotina.Authenticated",
                role="guillotina.Authenticated")
configure.grant(permission="guillotina.AddUser",
                role="guillotina.ContainerAdmin")
configure.grant(permission="guillotina.AddGroup",
                role="guillotina.ContainerAdmin")


def includeme(root, settings):
    configure.scan('guillotina_dbusers.content.users')
    configure.scan('guillotina_dbusers.content.groups')
    configure.scan('guillotina_dbusers.install')
    configure.scan('guillotina_dbusers.services')
    configure.scan('guillotina_dbusers.subscribers')



from . import users  # noqa
from guillotina import configure
from guillotina.api.content import DefaultPOST
from guillotina_dbusers.content.groups import IGroupManager
from guillotina_dbusers.content.users import IUserManager

# override some views...
configure.service(
    context=IGroupManager, method='POST', permission='guillotina.AddGroup'
)(DefaultPOST)


@configure.service(
    context=IUserManager, method='POST', permission='guillotina.AddUser')
class UserPOST(DefaultPOST):

    async def get_data(self):
        data = await super().get_data()
        if 'username' in data:
            data['id'] = data['username']
        elif 'id' in data:
            data['username'] = data['id']
        return data



# -*- encoding: utf-8 -*-
from guillotina import configure, schema
from guillotina.content import Folder
from guillotina.interfaces import IFolder
from guillotina_dbusers import _
from zope.interface import implementer


class IGroupManager(IFolder):
    pass


class IGroup(IFolder):

    name = schema.TextLine(
        title=_('Group name'),
        required=False)

    user_roles = schema.List(
        title=_('Roles'),
        value_type=schema.TextLine(),
        required=False
    )


@configure.contenttype(
    type_name="Group",
    schema=IGroup,
    add_permission="guillotina.AddGroup",
    behaviors=["guillotina.behaviors.dublincore.IDublinCore"])
class Group(Folder):
    name = None
    user_roles = []

    @property
    def roles(self):
        roles = {}
        for role in getattr(self, 'user_roles', []) or []:
            roles[role] = 1
        return roles

    @property
    def groups(self):
        return []

    @property
    def properties(self):
        return {}


@implementer(IGroupManager)
@configure.contenttype(
    type_name="GroupManager",
    schema=IGroupManager,
    behaviors=["guillotina.behaviors.dublincore.IDublinCore"],
    allowed_types=["Group"])
class GroupManager(Folder):
    pass



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

setup(
    name='guillotina_dbusers',
    version=open('VERSION').read().strip(),
    description='guillotina addon to provide users that are stored in the database',
    long_description=(open('README.rst').read() + '\n' +
                      open('CHANGELOG.rst').read()),
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords=['async', 'guillotina', 'users', 'auth'],
    author='Nathan Van Gheem',
    author_email='vangheem@gmail.com',
    url='https://pypi.python.org/pypi/guillotina_dbusers',
    license='GPL version 3',
    setup_requires=[
        'pytest-runner',
    ],
    zip_safe=True,
    include_package_data=True,
    packages=find_packages(exclude=['ez_setup']),
    install_requires=[
        'setuptools',
        'guillotina>=4.2.1'
    ],
    tests_require=[
        'pytest',
    ],
    extras_require={
        'test': [
            'pytest',
            'requests'
        ]
    }
)



from guillotina.testing import TESTING_SETTINGS
import json
import pytest


TESTING_SETTINGS.update({
    "auth_user_identifiers": [
        "guillotina_dbusers.users.DBUserIdentifier",
    ],
    "applications": [
        "guillotina_dbusers"
    ]
})


from guillotina.tests.conftest import *  # noqa
from guillotina.tests.conftest import ContainerRequesterAsyncContextManager  # noqa


class DBUsersRequester(ContainerRequesterAsyncContextManager):

    async def __aenter__(self):
        requester = await super().__aenter__()
        await requester('POST', '/db/guillotina/@addons', data=json.dumps({
             "id": "dbusers"
        }))
        return requester


@pytest.fixture(scope='function')
async def dbusers_requester(guillotina):
    return DBUsersRequester(guillotina)
